﻿<%@ WebHandler Language="VB" Class="register" %>
<%@ Assembly Src="src/util.vb" %>
<%@ Assembly Src="src/UserException.vb" %>
' The two "Assembly" lines does (virtually) nothing in IIS, but pacifies Mono.
' As this project is developed & debugged with IIS Express but deployed on a Ubuntu server with mod_mono,
' I have to make sure that I am not taking any advantage of IIS. (mod_mono is a subset of IIS)
' Not to mention, on my machine, despite on Windows, the preferred font is Ubuntu font :-) (which renders badly)

Imports System
Imports System.Web
Imports System.Text
Imports System.Security.Cryptography
Imports CE.iPhone.PList		   ' pList
Imports MySql.Data.MySqlClient ' MySql


Public Class register : Implements IHttpHandler
    
    Public Sub ProcessRequest(ByVal context As HttpContext) Implements IHttpHandler.ProcessRequest
        
		Dim db_conn As MySqlConnection = Nothing
		Dim plist_read As PListRoot = Nothing
		
		Try
			
			Dim data_read As PListDict
			
			Dim user As String
			Dim passHash As Byte()
			Dim email As String
			
			' Load data from upstream package
			If context.Request.InputStream.Length = 0 Then ' Check if there is any upoading data (i.e. POST)
				
				' Since there is no data in POST, try read the query string
				' Query strings are used as a mean of debugging with convenience
				data_read = New PListDict
				For Each key As String In context.Request.QueryString.AllKeys
					Dim value As String = context.Request.QueryString(key)
					If key = "password" Then ' Password need to be hashed first
						
						Dim buffer As Byte() = Encoding.UTF8.GetBytes(value)	 ' Get the string as binary...
						Dim bValue As Byte() = SHA512.Create.ComputeHash(buffer) ' ... and hash it
						
						data_read.Add(key, New PListData(bValue))
						
					Else
						
						data_read.Add(key, New PListString(value))
						
					End If
				Next
				
				plist_read = New PListRoot
				plist_read.Root = data_read ' Just to make a better crash report
				
				If data_read.Count = 0 Then _
					Throw New UserException("Request is missing.") ' Throws are heavily used as gotos here.
				
			Else
				
				Try
					
					' There is the uploading data, load the file assuming it is an plist
					plist_read = PListRoot.Load(context.Request.InputStream)
					data_read = CType(plist_read.Root, PListDict)
					
				Catch ex As Exception
					Throw New UserException("Request is not plist format. " +
											"Refer to the manual for the correct request format.", ex)
				End Try
				
			End If
			
			Try
				
				' Get data out of the plist (or the 
				user = CType(data_read("username"), PListString).Value	 ' Have to cast - .net is not as dynamic
				passHash = CType(data_read("password"), PListData).Value ' as Objective-C which have variables
				email = CType(data_read("email"), PListString).Value	 ' weakly typed and very loose messages.
				
			Catch ex As Exception
				
				Throw New UserException("Request is short of keys. " +
										"Refer to the manual for the correct request data.", ex)
				
			End Try
			
			' Load the database config file
			Dim dbcs = util.loadConfig(context.Server.MapPath("config.plist"))
			
			' Connect to the database
			db_conn = New MySqlConnection(dbcs)
			db_conn.Open()
			
			' Check if the user ID or email address already existed
			
			' SQL commands
			Dim select_uname As String = String.Format("SELECT * FROM {0} WHERE uname = '{1}';",
													   util.tableName("users"), MySqlHelper.EscapeString(user))
			Dim select_email As String = String.Format("SELECT * FROM {0} WHERE email = '{1}';",
													   util.tableName("users"), MySqlHelper.EscapeString(email))
			
			' SQL commands as objects to be invoked
			Dim db_cmd_uname As New MySqlCommand(select_uname, db_conn)
			Dim db_cmd_email As New MySqlCommand(select_email, db_conn)
			
			' Check user name
			Dim db_reader_uname As MySqlDataReader = db_cmd_uname.ExecuteReader()
			If db_reader_uname.HasRows() Then _
				Throw New UserException(String.Format("Username {0} already taken. Please try login", user))
			db_reader_uname.Close
			
			' Check Email
			Dim db_reader_email As MySqlDataReader = db_cmd_email.ExecuteReader()
			If db_reader_email.HasRows() Then _
				Throw New UserException(String.Format("Email {0} has an account linked to it. Please try login.",
						email))
			db_reader_email.Close
			
			' All OK, proceed to make an account
			Dim insert_user As String = String.Format("INSERT INTO {0} (`uname`, `pass_hash`, `email`) VALUES (@uname, @pass, @email);", util.tableName("users"))
			Dim db_cmd_insert As New MySqlCommand(insert_user, db_conn)
			db_cmd_insert.Parameters.Add(New MySqlParameter("@uname", MySqlDbType.String) With {.Value = user})
			db_cmd_insert.Parameters.Add(New MySqlParameter("@pass", MySqlDbType.Binary) With {.Value = passHash})
			db_cmd_insert.Parameters.Add(New MySqlParameter("@email", MySqlDbType.String) With {.Value = email})
			Dim return_val As Integer = db_cmd_insert.ExecuteNonQuery()
			
			' Since we have done everything, close the database connection.
			db_conn.Close()
			
			' Generate the output
			Dim data_out As New PListDict
			data_out.Add("status", New PListBool(True))
			data_out.Add("systemResult", New PListInteger(return_val))
			
			Dim plist_out As New PListRoot With {.Root = data_out}
			plist_out.Save(context.Response.OutputStream, PListFormat.Xml)
			
			context.Response.ContentType = "text/xml"
			
		Catch ex As UserException	' User request bad
			
			' Get the error information into a plist
			Dim data_err As New PListDict
			data_err.Add("errorMessage", New PListString(ex.Message))
			If Not IsNothing(plist_read) Then _
				data_err.Add("uploadData", plist_read.Root)
				
			' Present the error
			Dim plist_err As New PListRoot
			plist_err.Root = data_err
			plist_err.Save(context.Response.OutputStream, PListFormat.Xml)
				
			context.Response.ContentType = "text/xml"
			context.Response.StatusCode = 400 ' User error
			
		Catch ex As Exception ' Web app crashed.
			
			' Get the error information into a plist
			Dim data_err As New PListDict
			data_err.Add("errorMessage", New PListString(ex.Message))
			data_err.Add("errorDebug", New PListString(ex.StackTrace))
			data_err.Add("errorType", New PListString(ex.GetType.ToString()))
			If Not IsNothing(plist_read) Then _
				data_err.Add("uploadData", plist_read.Root)
			
			' Present the error to user
			Dim plist_err As New PListRoot
			plist_err.Root = data_err
			plist_err.Save(context.Response.OutputStream, PListFormat.Xml)
			
			context.Response.ContentType = "text/xml"
			context.Response.StatusCode = 500 ' Server crash
			
		Finally
			
			' Clean up
			If Not IsNothing(db_conn) AndAlso db_conn.State <> Data.ConnectionState.Closed Then _
				db_conn.Close()
			
		End Try
		
    End Sub
 
    Public ReadOnly Property IsReusable() As Boolean Implements IHttpHandler.IsReusable
        Get
            Return False
        End Get
    End Property

End Class